/*
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor,
    Boston, MA  02110-1301, USA.

    ---
    Copyright (C) 2006, Alex Wilson <nikosai@dragonhold.ath.cx>
 */

#include "main.h"
#include <iostream>

SudkVariable::SudkVariable(QFile *file, QObject *parent) : QObject(parent) {
	if (file->open(QIODevice::ReadOnly)) {
		QByteArray t = file->readAll();
		readFrom(t);
	}
}


void SudkVariable::readFrom(QByteArray source) {
	QDataStream in(&source, QIODevice::ReadOnly);

	char signature[9] = {'\0'};
	char folder[9] = {'\0'};
	char name[9] = {'\0'};
	quint32 size;
	quint16 datasize;
	quint16 checksum;

	in.readRawData(signature,8);
	in.skipRawData(2);
	in.readRawData(folder,8);
	in.skipRawData(40);
	in.skipRawData(6);
	in.readRawData(name,8);
	in.skipRawData(4);
	in >> size;
	in.skipRawData(6);
	in >> datasize;

	in.skipRawData(1);
	in.readRawData(_data,52);
	in >> checksum;

	_folder = QString(folder);
	_name = QString(name);
}

QByteArray SudkVariable::toWeb() {
	QByteArray temp;
	for (int i=0; i <= 81; i++) {
		if (get(i) > 0) temp.append(QString::number(get(i)).toAscii());
		else temp.append(".");
	}
	temp.append("\r\n");
	return temp;
}

QByteArray SudkVariable::toVBForums() {
	QByteArray temp;
	for (int row=1; row <=9; row++) {
		for (int col=1; col <= 9; col++) {
			if (get(row,col) > 0) temp.append(QString::number(get(row,col)).toAscii());
			else temp.append(".");
		}
		temp.append("\r\n");
	}
	return temp;
}

QByteArray SudkVariable::toSadMan() {
	QByteArray temp;
	temp.append("[Puzzle]\r\n");
	temp.append(toVBForums());
	return temp;
}

QByteArray SudkVariable::toSS() {
	QByteArray temp;
	for (int br=0; br <= 6; br+= 3) {
		for (int r=br; r<=br+2; r++) {
			for (int cb=0; cb<=6; cb+= 3) {
				for (int c=cb; c<=cb+2; c++) {
					if (get(r+1,c+1) > 0) temp.append(QString::number(get(r+1,c+1)).toAscii());
					else temp.append(".");
				}
				if (cb<6) temp.append("|");
			}
			temp.append("\r\n");
		}
		if (br<6) temp.append("-----------\r\n");
	}
	return temp;
}

QByteArray SudkVariable::toSPF() {
	QByteArray temp;
	for (int br=0; br <= 6; br+= 3) {
		for (int r=br; r<=br+2; r++) {
			for (int cb=0; cb<=6; cb+= 3) {
				for (int c=cb; c<=cb+2; c++) {
					temp.append(" ");
					if (get(r+1,c+1) > 0) temp.append(QString::number(get(r+1,c+1)).toAscii());
					else temp.append(".");
				}
				if (cb<6) temp.append(" |");
			}
			temp.append(" \r\n");
		}
		if (br<6) temp.append("-------+-------+------ \r\n");
	}
	return temp;
}

int SudkVariable::get(int index) {
	if (index % 2 == 0) {
		return (_data[index >> 1] & 240) >> 4;
	} else {
		return _data[index >> 1] & 15;
	}
}

int SudkVariable::get(int row, int column) {
	return get((row-1)*9 + (column-1));
}

CopyDialog::CopyDialog(QByteArray data, QWidget *parent) : QDialog(parent), _data(data) {
	setupUi(this);
	textEdit->setPlainText(QString(_data));
}

void MainWindow::on_actionWeb_activated() {
	if (_svar != NULL) {
		CopyDialog *dlg = new CopyDialog(_svar->toWeb(),this);
		dlg->show();
		connect(dlg, SIGNAL(accepted()), dlg, SLOT(deleteLater()));
	}
}

void MainWindow::on_actionVBForums_activated() {
	if (_svar != NULL) {
		CopyDialog *dlg = new CopyDialog(_svar->toVBForums(),this);
		dlg->show();
		connect(dlg, SIGNAL(accepted()), dlg, SLOT(deleteLater()));
	}
}

void MainWindow::on_actionSadMan_activated() {
	if (_svar != NULL) {
		CopyDialog *dlg = new CopyDialog(_svar->toSadMan(),this);
		dlg->show();
		connect(dlg, SIGNAL(accepted()), dlg, SLOT(deleteLater()));
	}
}

void MainWindow::on_actionSS_activated() {
	if (_svar != NULL) {
		CopyDialog *dlg = new CopyDialog(_svar->toSS(),this);
		dlg->show();
		connect(dlg, SIGNAL(accepted()), dlg, SLOT(deleteLater()));
	}
}


void MainWindow::on_actionSPF_activated() {
	if (_svar != NULL) {
		CopyDialog *dlg = new CopyDialog(_svar->toSPF(),this);
		dlg->show();
		connect(dlg, SIGNAL(accepted()), dlg, SLOT(deleteLater()));
	}
}

MainWindow::MainWindow(QWidget *parent) : QMainWindow(parent) {
	setupUi(this);
}

void MainWindow::on_actionQuit_activated() {
	QApplication::quit();
}

MainWindow::~MainWindow() {
	if (_svar != NULL) delete _svar;
}

void MainWindow::on_actionOpen_activated() {
	QString fname = QFileDialog::getOpenFileName(this, "Choose a file", "", "TI Application Variables (*.89y *.92y *.v2y)");
	if (fname != "") {
		if (_svar != NULL) delete _svar;
		QFile f(fname);
		_svar = new SudkVariable(&f);
		refreshTable();
	}
}

void CopyDialog::on_saveButton_clicked() {
	QString fname = QFileDialog::getSaveFileName(this, "Choose a file to save", "",
		"Web sudoku format (*.txt);;VB Forums format (*.msk *.sol);;SadMan Sudoku format (*.sdk);;Simple Sudoku (*.ss);;SuDoku Solver format (*.spf)");
	if (fname != "") {
		QFile f(fname);
		if (f.open(QIODevice::WriteOnly)) {
			//qDebug() << datatowrite;
			f.write(_data);
			f.close();
		}
	}
}

void MainWindow::refreshTable() {
	QTableWidgetItem *item;
	if (_svar != NULL) {
		for (int row=1; row<=9; row++) {
			tableWidget->setColumnWidth(row-1,25);
			tableWidget->setRowHeight(row-1,25);
			for (int col=1; col<=9; col++) {
				item = tableWidget->item(row-1,col-1);
				if (item) {
					if (_svar->get(row,col) > 0) item->setText(QString::number(_svar->get(row,col)));
					else item->setText(".");
				} else {
					item = new QTableWidgetItem(QString::number(_svar->get(row,col)));
					if (_svar->get(row,col) == 0) item->setText(".");
					tableWidget->setItem(row-1, col-1, item);
				}
			}
		}
	}
}

int main(int argc, char *argv[]) {
	QApplication *app = new QApplication(argc, argv);

	MainWindow *mw = new MainWindow();
	mw->show();

	return app->exec();
}
